
# q-expansion of newform 6600.2.a.s, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6600
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [-3], [1], [0], [3], [3], [3], [-6], [-10], [-7], [7], [4], [9], [12], [7], [-4], [12], [-15], [4], [15], [6], [0], [-7], [-17], [-2], [-10], [-4], [4], [11], [14], [2], [12], [9], [24], [18], [-16], [-8], [9], [19], [1], [1], [4], [21], [20], [-20], [16], [14], [-3], [9], [0], [-10], [0], [22], [10], [-6], [-5], [-8], [11], [-17], [-21], [-4], [-24], [1], [0], [24], [24], [12], [26], [8], [10], [26], [2], [-14], [8], [22], [-14], [-24], [12], [1], [19], [-18], [30], [27], [-15], [-24], [10], [-34], [14], [-36], [12], [2], [2], [-24], [24], [-20], [-42], [1], [-8], [17], [2], [26], [17], [32], [-37], [-27], [26], [-13], [0], [24], [-32], [-18], [42], [-10], [-32], [10], [-19], [6], [-8], [25], [-36], [-38], [-41], [-12], [33], [-35], [-40], [-8], [-8], [25], [-8], [-44], [-2], [-30], [44], [-4], [33], [-10], [-39], [35], [2], [56], [-16], [-14], [0], [42], [-21], [-4], [40], [-14], [4], [-26], [-36], [58], [33], [-25], [16], [-2], [-13], [-31], [3], [-48], [41], [-2], [-27], [48], [14], [-36], [-24], [-38], [14], [6], [-34], [50], [-44], [-41], [-2], [-45], [30], [-54], [26], [-19], [26], [32], [-42], [-26], [36], [-10], [22], [-62], [36], [26], [-42], [-28], [-30], [-32], [30], [-17], [48], [-39], [-16], [-37], [56], [-21], [-34], [6], [42], [45], [44], [-18], [-48], [-10], [-45], [-38], [-13], [-15], [-25], [-29], [38], [50], [40], [-24], [38], [-54], [-46], [68], [-19], [-31], [72], [61], [-7], [-75], [-19], [-68], [36], [31], [11], [-19], [21], [-17], [-37], [-44], [-71], [9], [-48], [-32], [31], [-72], [36], [38], [-5], [-68], [-6], [42], [66], [51], [-32], [26], [-72], [49], [30], [40], [-26], [26], [-10], [82], [6], [5], [49], [65], [4], [-14], [43], [10], [-38], [47], [66], [-38], [28], [-66], [32], [-36], [-34], [60], [-3], [-23], [57], [-70], [23], [40], [63], [57], [50], [74], [18], [28], [42], [12], [39], [-46], [75], [21], [-44], [36], [22], [-67], [-37], [33], [-32], [-62], [-36], [59], [-58], [-74], [-40], [-21], [-42], [-67], [78], [1], [-10], [-51], [26], [-30], [85], [-10], [86], [-30], [-39], [-28], [-26], [-17], [6], [-73], [-49], [14], [42], [29], [77], [35], [-48], [30], [-12], [-14], [-44], [-39], [22], [-80], [52], [-91], [-37], [34], [29], [-68], [-73], [-49], [-12], [-85], [-32], [-26], [51], [-33], [48], [76], [67], [31], [64], [-80], [60], [40], [58], [75], [23], [-11], [-24], [-54], [16], [-30], [-40], [36], [48], [-11], [-62], [8], [41], [-2], [45], [-25], [80], [95], [-8], [27], [-18], [-16], [40], [-33], [54], [-22], [-61], [-80], [92], [72], [18], [77], [37], [6], [-30], [-8], [-7], [78], [-77], [51], [57], [38], [-30], [-64], [-78], [-98], [-78], [46], [98], [-3], [58], [-59], [0], [-69], [-77], [-90], [58]]
